@@ -51,7 +51,7 @@ let string_of_kind = function
 
 let rec longident_of_path = function
     Pident id -> Lident (Ident.name id)
-  | Pdot (path, s, _) -> Ldot (longident_of_path path, s)
+  | Pdot (path, s) -> Ldot (longident_of_path path, s)
   | Papply (p1, p2) -> Lapply (longident_of_path p1, longident_of_path p2)
 
 let rec remove_prefix lid ~prefix =
@@ -201,11 +201,12 @@ let mklid = function
 let mkpath = function
     [] -> raise (Invalid_argument "Searchid.mklid")
   | x :: l ->
-      List.fold_left l ~init:(Pident (Ident.create x))
-      ~f:(fun acc x -> Pdot (acc, x, 0))
+      List.fold_left l ~init:(Pident (Ident.create_local x))
+      ~f:(fun acc x -> Pdot (acc, x))
 
 let get_fields ~prefix ~sign self =
-  let env = open_signature Fresh (mkpath prefix) sign !start_env in
+  (*let env = open_signature Fresh (mkpath prefix) sign !start_env in*)
+  let env = add_signature sign !start_env in
   match (expand_head env self).desc with
     Tobject (ty_obj, _) ->
       let l,_ = flatten_fields ty_obj in l
@@ -222,9 +223,9 @@ let rec search_type_in_signature t ~sign ~prefix ~mode =
   in
   List2.flat_map sign ~f:
   begin fun item -> match item with
-        Sig_value (id, vd) ->
+        Sig_value (id, vd, _) ->
           if matches vd.val_type then [lid_of_id id, Pvalue] else []
-      | Sig_type (id, td, _) ->
+      | Sig_type (id, td, _, _) ->
           if
           matches (newconstr (Pident id) td.type_params) ||
           begin match td.type_manifest with
@@ -244,23 +245,23 @@ let rec search_type_in_signature t ~sign ~prefix ~mode =
             List.exists l ~f:(fun {Types.ld_type=t} -> matches t)
           end
           then [lid_of_id id, Ptype] else []
-      | Sig_typext (id, l, _) ->
+      | Sig_typext (id, l, _, _) ->
           if constructor_matches l.ext_args
           then [lid_of_id id, Pconstructor]
           else []
-      | Sig_module (id, {md_type=Mty_signature sign}, _) ->
+      | Sig_module (id, _, {md_type=Mty_signature sign}, _, _) ->
           search_type_in_signature t ~sign ~mode
             ~prefix:(prefix @ [Ident.name id])
       | Sig_module _ -> []
       | Sig_modtype _ -> []
-      | Sig_class (id, cl, _) ->
+      | Sig_class (id, cl, _, _) ->
           let self = self_type cl.cty_type in
           if matches self
           || (match cl.cty_new with None -> false | Some ty -> matches ty)
           (* || List.exists (get_fields ~prefix ~sign self)
               ~f:(fun (_,_,ty_field) -> matches ty_field) *)
           then [lid_of_id id, Pclass] else []
-      | Sig_class_type (id, cl, _) ->
+      | Sig_class_type (id, cl, _, _) ->
           let self = self_type cl.clty_type in
           if matches self
           (* || List.exists (get_fields ~prefix ~sign self)
@@ -273,12 +274,13 @@ let search_all_types t ~mode =
       `Exact, _ -> [t]
     | `Included, Tarrow _ -> [t]
     | `Included, _ ->
-      [t; newty(Tarrow(Nolabel,t,newvar(),Cok)); newty(Tarrow(Nolabel,newvar(),t,Cok))]
+      [t; newty(Tarrow(Nolabel,t,newvar(),Cok));
+          newty(Tarrow(Nolabel,newvar(),t,Cok))]
   in List2.flat_map !module_list ~f:
     begin fun modname ->
     let mlid = Lident modname in
-    try match find_module (lookup_module ~load:true mlid !start_env) !start_env
-    with {md_type=Mty_signature sign} ->
+    try match Env.find_module_by_name mlid !start_env
+    with _, {md_type=Mty_signature sign} ->
         List2.flat_map tl
           ~f:(search_type_in_signature ~sign ~prefix:[modname] ~mode)
     | _ -> []
@@ -307,7 +309,7 @@ let search_string_type text ~mode =
           let end_c = l.loc_end.Lexing.pos_cnum in
           raise (Error (start_c - 8, end_c - 8))
     in match sign with
-        [ Sig_value (_, vd) ] ->
+        [ Sig_value (_, vd, _) ] ->
           search_all_types vd.val_type ~mode
       | _ -> []
   with
@@ -359,23 +361,21 @@ let search_pattern_symbol text =
   let check i = check_match ~pattern (explode (Ident.name i)) in
   let l = List.map !module_list ~f:
     begin fun modname -> Lident modname,
-    try match
-      find_module (lookup_module ~load:true (Lident modname) !start_env)
-	!start_env
-    with {md_type=Mty_signature sign} ->
+    try match find_module_by_name (Lident modname) !start_env with
+    | _, {md_type=Mty_signature sign} ->
         List2.flat_map sign ~f:
           begin function
-            Sig_value (i, _) when check i -> [i, Pvalue]
-          | Sig_type (i, _, _) when check i -> [i, Ptype]
-          | Sig_typext (i, _, _) when check i -> [i, Pconstructor]
-          | Sig_module (i, _, _) when check i -> [i, Pmodule]
-          | Sig_modtype (i, _) when check i -> [i, Pmodtype]
-          | Sig_class (i, cl, _) when check i
+            Sig_value (i, _, _) when check i -> [i, Pvalue]
+          | Sig_type (i, _, _, _) when check i -> [i, Ptype]
+          | Sig_typext (i, _, _, _) when check i -> [i, Pconstructor]
+          | Sig_module (i, _, _, _, _) when check i -> [i, Pmodule]
+          | Sig_modtype (i, _, _) when check i -> [i, Pmodtype]
+          | Sig_class (i, cl, _, _) when check i
             || List.exists
                 (get_fields ~prefix:[modname] ~sign (self_type cl.cty_type))
                 ~f:(fun (name,_,_) -> check_match ~pattern (explode name))
             -> [i, Pclass]
-          | Sig_class_type (i, cl, _) when check i
+          | Sig_class_type (i, cl, _, _) when check i
             || List.exists
                 (get_fields ~prefix:[modname] ~sign (self_type cl.clty_type))
                 ~f:(fun (name,_,_) -> check_match ~pattern (explode name))
@@ -406,13 +406,13 @@ let search_string_symbol text =
     try let _ = f lid !start_env in [lid, k]
     with Not_found | Env.Error _ -> []
   in
-  try_lookup lookup_constructor Pconstructor @
-  try_lookup (lookup_module ~load:true) Pmodule @
-  try_lookup lookup_modtype Pmodtype @
-  try_lookup lookup_value Pvalue @
-  try_lookup lookup_type Ptype @
-  try_lookup lookup_label Plabel @
-  try_lookup lookup_class Pclass
+  try_lookup find_constructor_by_name Pconstructor @
+  try_lookup find_module_by_name Pmodule @
+  try_lookup find_modtype_by_name Pmodtype @
+  try_lookup find_value_by_name Pvalue @
+  try_lookup find_type_by_name Ptype @
+  try_lookup find_label_by_name Plabel @
+  try_lookup find_class_by_name Pclass
 
 open Parsetree
 
@@ -448,7 +448,7 @@ let search_structure str ~name ~kind ~prefix =
           List.fold_left ~init:[] str ~f:
             begin fun acc item ->
               match item.pstr_desc with
-                Pstr_module x when x.pmb_name.txt = modu ->
+                Pstr_module x when x.pmb_name.txt = Some modu ->
                   loc := x.pmb_expr.pmod_loc.loc_start.Lexing.pos_cnum;
                   begin match x.pmb_expr.pmod_desc with
                     Pmod_structure str -> str
@@ -483,8 +483,9 @@ let search_structure str ~name ~kind ~prefix =
 	      then loc := td.pext_loc.loc_start.Lexing.pos_cnum
             end;
           false
-      | Pstr_exception pcd when kind = Pconstructor -> name = pcd.pext_name.txt
-      | Pstr_module x when kind = Pmodule -> name = x.pmb_name.txt
+      | Pstr_exception pcd when kind = Pconstructor ->
+          name = pcd.ptyexn_constructor.pext_name.txt
+      | Pstr_module x when kind = Pmodule -> Some name = x.pmb_name.txt
       | Pstr_modtype x when kind = Pmodtype -> name = x.pmtd_name.txt
       | Pstr_class l when kind = Pclass || kind = Ptype || kind = Pcltype ->
           List.iter l ~f:
@@ -509,14 +510,14 @@ let search_signature sign ~name ~kind ~prefix =
   ignore (name = "");
   ignore (prefix = [""]);
   let loc = ref 0 in
-  let rec search_module_type  sign ~prefix =
+  let rec search_module_type sign ~prefix =
     match prefix with [] -> sign
     | modu::prefix ->
         let sign =
           List.fold_left ~init:[] sign ~f:
             begin fun acc item ->
               match item.psig_desc with
-                Psig_module pmd when pmd.pmd_name.txt = modu ->
+                Psig_module pmd when pmd.pmd_name.txt = Some modu ->
                   loc := pmd.pmd_type.pmty_loc.loc_start.Lexing.pos_cnum;
                   begin match pmd.pmd_type.pmty_desc with
                     Pmty_signature sign -> sign
@@ -544,8 +545,9 @@ let search_signature sign ~name ~kind ~prefix =
 	      then loc := td.pext_loc.loc_start.Lexing.pos_cnum
             end;
           false
-      | Psig_exception pcd when kind = Pconstructor -> name = pcd.pext_name.txt
-      | Psig_module pmd when kind = Pmodule -> name = pmd.pmd_name.txt
+      | Psig_exception pcd when kind = Pconstructor ->
+          name = pcd.ptyexn_constructor.pext_name.txt
+      | Psig_module pmd when kind = Pmodule -> Some name = pmd.pmd_name.txt
       | Psig_modtype pmtd when kind = Pmodtype -> name = pmtd.pmtd_name.txt
       | Psig_class l when kind = Pclass || kind = Ptype || kind = Pcltype ->
           List.iter l ~f:
